Package: arrayQuality
Version: 1.89.0
Date: 2010-04-13
Title: Assessing array quality on spotted arrays
Depends: R (>= 2.2.0)
Imports: graphics, grDevices, grid, gridBase, hexbin, limma, marray,
        methods, RColorBrewer, stats, utils
biocViews: Microarray,TwoChannel,QualityControl,Visualization
Suggests: mclust, MEEBOdata, HEEBOdata
Author: Agnes Paquet and Jean Yee Hwa Yang <yeehwa@stat.berkeley.edu>
Maintainer: Agnes Paquet <paquetagnes@yahoo.com>
Description: Functions for performing print-run and array level
        quality assessment.
License: LGPL
URL: http://arrays.ucsf.edu/
git_url: https://git.bioconductor.org/packages/arrayQuality
git_branch: devel
git_last_commit: 354cf88
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 20:03:03 UTC; biocbuild
